# SSL onlyoffice avec NGINX 

### etape 1

Enlever le  fichier de conf par default 

```shell
unlink /etc/nginx/sites-enabled/default
```

### etape 2

Creer le nouveau fichier de config : 
```shell
nano /etc/nginx/sites-available/onlyoffice.conf
```

```nginx
server {
    listen 80;
    listen [::]:80;
    server_name onlyoffice.mondomaine.fr;

    # reverse proxy
    location / {
                proxy_pass http://127.0.0.1:4242;
    }
}
```

### etape 3


Activer la nouvelle configuration
```shell
ln -s /etc/nginx/sites-available/onlyoffice.conf /etc/nginx/sites-enabled/onlyoffice.conf
```

tester la configuration
```shell
nginx -t
```

passer la configuration en prod 
```shell
service nginx restart
```

### etape 4
Creer le fichier de config pour le ssl: 
```shell
nano /etc/nginx/sites-available/onlyoffice-ssl.conf
```

```nginx
server {
    listen 443 ssl;
    server_name onlyoffice.mondomaine.fr;

    # option ssl
    ssl on;
    ssl_protocols TLSv1.3 TLSv1.2;
    ssl_ciphers "HIGH:!aNULL:!MD5:!ADH:!DH:!RC4:!RSA";
    ssl_prefer_server_ciphers on;

    # certificats
    ssl_certificate /etc/nginx/ssl/ardenne.cer;
    ssl_certificate_key /etc/nginx/ssl/ardenne-metropole_2022.key;

    # si le certificat demande un mot de passe, stocker le mdp dans le fichier suivant
    ssl_password_file /etc/nginx/ssl/global.pass;

    # reverse proxy avec forward sur le proto https
    location / {
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;

        proxy_pass http://127.0.0.1:4242;
        proxy_redirect http://127.0.0.1:4242 https://onlyoffice.mondomaine.fr
    }
}
```

### etape 5

Activer la nouvelle configuration
```shell
ln -s /etc/nginx/sites-available/onlyoffice-ssl.conf /etc/nginx/sites-enabled/onlyoffice-ssl.conf
```

tester la configuration
```shell
nginx -t
```

passer la configuration en prod 

```shell
service nginx restart
```