**Paramèttrage de la base de données PosgreSQL** 

L'utilisateur de la base de données ne doit avoir que des droits SELECT sur la base de données et les tables à interroger. 
Grafana ne valide pas la sécurité des requêtes. Celles-ci peuvent donc contenir n'importe quelle instruction SQL. 
Par exemple, des instructions telles que DELETE FROM user ; et DROP TABLE user ; seraient exécutées. 
Pour vous protéger, nous vous recommandons vivement de créer un utilisateur PostgreSQL spécifique avec des droits restreints.

* Création d'un utilisateur en lecture seule sur la base de données Maarch Courrier

    1. Pour créer un nouvel utilisateur dans PostgreSQL :
    
            su postgres
            psql
            CREATE USER grafana WITH ENCRYPTED PASSWORD 'user_password';
            
    2. Accordez l'accès à la base de données :
    
            GRANT CONNECT ON DATABASE database_name TO grafana;
            
    3. Puis accordez un droit d'utilisation sur le schéma de la base:
    
            GRANT USAGE ON SCHEMA schema_name TO grafana;
    
    4. accordez le droit SELECT sur les tables ou l'ensemble des tables de la base de données:
    
            GRANT SELECT ON table_name TO grafana;
            GRANT SELECT ON ALL TABLES IN SCHEMA schema_name TO grafana;
        
        - liste des tables concernées
        
        | **Nom de la table**          | **Description**                        |
        |--------------------------|------------------------------------|
        | acknowledgement_receipts | Table des accusés de réception     |
        | doctypes                 | Types de documents                 |
        | entities                 | Table des entites                  |
        | indexing_models          | Table des modèles d'enregistrement |
        | priorities               | Table des priorités                |
        | res_letterbox            | Table des courriers                |
        | status                   | Tables des statuts                 |
        
        - Pour plus d'information sur les colonnes des tables, [cliquez ici !](https://docs.maarch.org/MaarchCourrier/guat/guat_architecture/database_model.html)
        
**Installation de grafana et preréquis**

* Grafana community, [cliquez ici !](https://grafana.com/docs/grafana/latest/installation/docker/) ou Grafana entreprise (recommandée) , [cliquez ici !](https://grafana.com/grafana/download?edition=enterprise)

* Création de votre source de données: [cliquez ici !](https://docs.maarch.org/MaarchCourrier/guat/guat_exploitation/docker-grafana.html)
    
* installer le panel datatable, [le lien d'installation:](https://grafana.com/grafana/plugins/briangann-datatable-panel/?tab=installation)

* installer le panel bar-chart, [le lien d'installation:](https://grafana.com/docs/grafana/latest/panels/visualizations/bar-chart/) (optionel)
   
* installer le plugin Grafana Image Renderer , [le lien d'installation:](https://grafana.com/grafana/plugins/grafana-image-renderer/ (exporter les panels en images))
  
**L'import du dashboard**
    
* Télécharger le fichier dashboard.json [cliquez ici !](https://labs.maarch.org/maarch/howto.doc/blob/1.0/grafana/dashboard.json)
* Pour plus d'information sur l'import, [cliquez ici !](https://docs.maarch.org/MaarchCourrier/guat/guat_exploitation/docker-grafana.html)

**Liaison de la soucre de données au dashboard**

Afin de lier votre source de données ( base de données ) au tableau de bord, vous pouvez:

- Cocher la case Default dans les paramètres de la source de données afin qu'elle soit sélectionnée par défaut 
- Séléctionner pour chaque panel ( panneau ) la source de données Maarch Courrier 
    
![](images/dataSourceSelect.png)

**Liaison de la soucre de données aux variables**

![](images/var_datasource.png)

A faire sur toutes les variables

**Manipulation de requêtes**

![](images/requestAdmin.png)

Pour plus de détails [cliquez ici !](https://grafana.com/docs/grafana/latest/datasources/postgres/)

**Export du tableau de bord**

Afin d'exporter les graphiques du tableau de bord, vous avez deux choix:

- Exoprter un pannel au format png (nécessite l'installation du plugin Grafana Image Renderer)

![](images/pngExport.png)

- Exporter le tableau de bord au format PDF 

    Pour plus d'information, [cliquez ici !](https://labs.maarch.org/maarch/howto.doc/blob/1.0/grafana/grafana_export/grafana_export.md)

**Administration des droits: Utilisateurs, Repertoires, tableau de bord...**

 - [Administration des utilisateurs ](https://grafana.com/docs/grafana/latest/manage-users/)
 
 - [Administration des permissions ](https://grafana.com/docs/grafana/latest/permissions/)

**Liste des requêtes SQL associées au tableau de bord**

Pour plus d'information, [cliquez ici !](https://labs.maarch.org/maarch/howto.doc/blob/1.0/grafana/requests.md)

**Quelques liens utiles**

 
 [Paramètrage de grafana ](https://grafana.com/docs/grafana/latest/administration/)
 
 [Administration des préférences ](https://grafana.com/docs/grafana/latest/administration/preferences/)
 
 [Administration des data sources ](https://grafana.com/docs/grafana/latest/datasources/postgres/)
 
 [Administration des panneaux ](https://grafana.com/docs/grafana/latest/panels/)
 
 [Administration des tableau de bord ](https://grafana.com/docs/grafana/latest/dashboards/)
 
 [Partager un tableau de bord ](https://grafana.com/docs/grafana/latest/sharing/share-dashboard/)
 
 [Création de liens ](https://grafana.com/docs/grafana/latest/linking/)
